/***********************************************************************
 *
 * linux/arch/arm/mach-ns9750/arch.c
 *
 * Copyright (C) 2006 Atmark Techno, Inc.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 *
 ***********************************************************************/
#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/ioport.h>
#include <linux/module.h>
#include <linux/device.h>

#include <asm/io.h>

#include <asm/mach-types.h>
#include <asm/mach/arch.h>
#include <asm/mach/map.h>
#include <asm/mach/flash.h>

#include <asm/arch/ns9750_config.h>
#include <asm/arch/ns9750_sys.h>
#include <asm/arch/ns9750_bbus.h>
#include <asm/arch/ns9750_bbus_bridge.h>
#include <asm/arch/ns9750_mem.h>
#include <asm/arch/ns9750_usb.h>
#include <asm/arch/irqs.h>
#include <asm/arch/time.h>

#if (VMALLOC_END >= 0xf8000000)
/* Documentation/arm/memory.txt leaves platform space from
   from VMALLOC_END to 0xff000000. */
# error VMALLOC_END clashes with peripheral pages
#endif

#if defined(CONFIG_MACH_ARMADILLO300)
static struct map_desc standard_io_desc[] __initdata = {
	/* virtual     physical    length      type */
	{ 0xfa900000, 0xa0900000, 0x00100000, MT_DEVICE }, /*SCM*/
	{ 0xfa700000, 0xa0700000, 0x00100000, MT_DEVICE }, /*MEM*/
	{ 0xfa600000, 0xa0600000, 0x00100000, MT_DEVICE }, /*ETH*/
	{ 0xfa400000, 0xa0400000, 0x00100000, MT_DEVICE }, /*BBB*/
	{ 0xf9000000, 0x90000000, 0x00200000, MT_DEVICE }, /*DMAUSB*/
	{ 0xf9600000, 0x90600000, 0x00100000, MT_DEVICE }, /*BBU*/
	{ 0xf9500000, 0x90500000, 0x00100000, MT_DEVICE }, /*I2C*/
	{ 0xfa800000, 0xa0800000, 0x00100000, MT_DEVICE }, /*LCD*/
	{ 0xf9200000, 0x90200000, 0x00200000, MT_DEVICE }, /*SER*/
	{ 0xf9400000, 0x90400000, 0x00100000, MT_DEVICE }, /*BSD*/
	{ 0xfa000000, 0xa0000000, 0x00100000, MT_DEVICE }, /*PCI_IO*/
	{ 0xfa100000, 0xa0100000, 0x00100000, MT_DEVICE }, /*PCI_CA*/
	{ 0xfa200000, 0xa0200000, 0x00100000, MT_DEVICE }, /*PCI_CD*/
	{ 0xfa300000, 0xa0300000, 0x00100000, MT_DEVICE }, /*PCI*/
	{ 0xf6000000, 0x60000000, 0x00100000, MT_DEVICE }, /*MISCREG*/
	{ 0xf7000000, 0x62000000, 0x00100000, MT_DEVICE }, /*IDE*/
};
#endif

static struct resource resources[] = {
	{
		.name	= "bbus_utility",
		.start	= NS_BBUS_UTILITY_BASE,
		.end	= (NS_BBUS_UTILITY_BASE + NS_BBUS_ARM_WAKE_UP + 3)
	}, {
		.name	= "dma_0",
		.start	= (NS_BBUS_DMA_DESCR( 0, 0 )),
		.end	= (NS_BBUS_DMA_STAT( 0, 15 ) + 3)
	}, {
		.name	= "dma_1",
		.start	= (NS_BBUS_DMA_DESCR( 1, 0 )),
		.end	= (NS_BBUS_DMA_STAT( 1, 15 ) + 3)
	}, {
		.name = NULL
	}
};

static struct resource ns9750_usbhost_resources [] = {
        [0] = {
                .start  = (NS_USB_HCR),
                .end    = (NS_USB_HCR_PORT_SAT),
                .flags  = IORESOURCE_MEM,
        },
	[1] = {
		.start = (IRQ_USB_HOST),
		.end = (IRQ_USB_HOST),
		.flags = IORESOURCE_IRQ,
	},
};

static u64 ns9750_usbhost_dma_mask = 0xffffffffUL;

static struct platform_device ns9750_usbhost_device = {
        .name           = "ns9750-ohci",
        .id             = 0,
        .dev            = {
                .dma_mask = &ns9750_usbhost_dma_mask,
                .coherent_dma_mask = 0xffffffffUL,
        },
        .num_resources  = ARRAY_SIZE (ns9750_usbhost_resources),
        .resource       = ns9750_usbhost_resources,
};

static struct resource ns9750_ethernet_resources[] = {
	[0] = {
	  .start	= 0xa0600000,//HwRegToPhys(MAC_BASE),
	  .end	= 0xa0600000 + 0xffff,//HwRegToPhys(MAC_BASE) + 0xffff,
	  .flags	= IORESOURCE_MEM,
	},
	[1] = {
	  .start	= IRQ_ETH_RX,//IRQ_MAC,
	  .end	= IRQ_ETH_PHY,//IRQ_MAC,
	  .flags	= IORESOURCE_IRQ,
	},
};

static struct platform_device ns9750_ethernet_device = {
	.name		= "ns9750ethernet",
	.id		= 0,
	.num_resources	= 2,
	.resource	= ns9750_ethernet_resources,
};

static struct resource ns9750_raster_resources[] = {
  [0] = {
    .start	= 0xa0800000,
    .end	= 0xa0800000 + 0x0fffff, 
    .flags	= IORESOURCE_MEM,
  },
};

static struct platform_device ns9750_raster_device = {
  .name	= "ns9750fb",
  .id	= 0,
  .resource	= ns9750_raster_resources,
  .num_resources	= 1,
};

static struct flash_platform_data armadillo3x0_flash_data = {
        .map_name       = "cfi_probe",
        .width          = 2,
};

static struct resource armadillo3x0_flash_resource = {
        .start          = 0x50000000,
        .end            = 0x507fffff,
        .flags          = IORESOURCE_MEM,
};

static struct platform_device armadillo3x0_flash_device = {
	.name		= "armadillo3x0-nor",
	.id		= 0,
	.dev		= {
		.platform_data	= &armadillo3x0_flash_data,
	},
	.num_resources	= 1,
	.resource	= &armadillo3x0_flash_resource,
};

static struct platform_device *ns9750_devices[] __initdata = {
	&ns9750_ethernet_device,
	&ns9750_usbhost_device,
	&ns9750_raster_device,
};

static struct platform_device *armadillo3x0_devices[] __initdata = {
	&armadillo3x0_flash_device,
};

static unsigned int _ns_sys_clock_freq;

spinlock_t ns_gpio_lock;	/* use it to lock all GPIO accesses */

extern void __init ns_init_irq( void );
static void __init ns_map_io( void );
static int __init ns_init( void );

static void __init ns_map_io(void)
{
	_ns_sys_clock_freq = NS_SYS_CLK_FREQ;

	iotable_init( standard_io_desc, ARRAY_SIZE( standard_io_desc ) );

	printk( "NS9750 Rev. %i, running at %u MHz\n",
		(readl( NS_SYS_MISC ) >> 24) & 0xff,
		_ns_sys_clock_freq/2000000 );
}

static int __init ns_init( void )
{
	int i = 0;

	platform_add_devices(ns9750_devices, ARRAY_SIZE(ns9750_devices));
#if defined(CONFIG_MACH_ARMADILLO300)
	platform_add_devices(armadillo3x0_devices, ARRAY_SIZE(armadillo3x0_devices));
#endif

	spin_lock_init( &ns_gpio_lock );

	while( resources[ i ].name != NULL ) {
		request_resource( &iomem_resource, &resources[ i++ ] );
	}

	return 0;
}

/***********************************************************************
 * @Function:
 * @Return:
 * @Descr:  provides system clock for drivers
 ***********************************************************************/

unsigned int ns_sys_clock_freq( void )
{
	return _ns_sys_clock_freq;
}

EXPORT_SYMBOL( ns_sys_clock_freq );

arch_initcall( ns_init );

#if defined(CONFIG_MACH_ARMADILLO300)
MACHINE_START(ARMADILLO300, "ARMADILLO300")
     MAINTAINER("Atmark Techno, Inc.")
     BOOT_MEM(0x00000000, 0x90000000, 0xf9000000)
     BOOT_PARAMS(0x00000100)
     MAPIO(ns_map_io)
     INITIRQ(ns_init_irq)
     .timer = &ns_timer,
MACHINE_END
#endif
