/*
 *  linux/include/asm-arm/arch-ns9750/irqs.h
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __ASM_ARM_ARCH_IRQS_H
#define __ASM_ARM_ARCH_IRQS_H

/* AHB Interrupt numbers, [1], Chap. 4 */

#define IRQ_WATCHDOG		0
#define IRQ_AHB_BUS_ERROR	1
#define IRQ_BBUS_AGGREGATE	2 /* @Footnote 1 */
#define IRQ_RESERVED		3
#define IRQ_ETH_RX  		4
#define IRQ_ETH_TX		5
#define IRQ_ETH_PHY		6
#define IRQ_ETH			4
#define IRQ_ETH_RX_OFFSET	0 /* RX  = ETH + OFFSET(0) */
#define IRQ_ETH_TX_OFFSET	1 /* TX  = ETH + OFFSET(1) */
#define IRQ_ETH_PHY_OFFSET	2 /* PHY = ETH + OFFSET(2) */
#define IRQ_LCD			7
#ifdef CONFIG_ARCH_NS9360
#define IRQ_UARTB_RX		8
#define IRQ_UARTB_TX		9
#define IRQ_UARTA_RX		10
#define IRQ_UARTA_TX		11
#define IRQ_UARTC_RX		12
#define IRQ_UARTC_TX		13
#else
#define IRQ_PCI_BRIDGE		8
#define IRQ_PCI_ARBITER		9
#define IRQ_PCI_EXT_0		10
#define IRQ_PCI_EXT_1		11
#define IRQ_PCI_EXT_2		12
#define IRQ_PCI_EXT_3		13
#endif
#define IRQ_I2C     		14
#define IRQ_BBUS_DMA		15
#define IRQ_TIMER_0		16
#define IRQ_TIMER_1		17
#define IRQ_TIMER_2		18
#define IRQ_TIMER_3		19
#define IRQ_TIMER_4		20
#define IRQ_TIMER_5		21
#define IRQ_TIMER_6		22
#define IRQ_TIMER_7		23
#ifdef CONFIG_ARCH_NS9360
#define IRQ_RTC			24
#define IRQ_USB_HOST_1		25
#define IRQ_USB_DEVICE		26
#define IRQ_1284		27
#else
#define IRQ_TIMER_8_9		24
#define IRQ_TIMER_10_11		25
#define IRQ_TIMER_12_13		26
#define IRQ_TIMER_14_15		27
#endif
#define IRQ_EXT_0		28
#define IRQ_EXT_1		29
#define IRQ_EXT_2		30
#define IRQ_EXT_3		31

#define NR_IRQ_BBUS_START	(IRQ_EXT_3+1)

/* BBUS Interrupts [1], chap 8 */
#define IRQ_BBUS_DMA_AGG	(NR_IRQ_BBUS_START) /* @Footnote 1 */
#define IRQ_BBUS_USB		(NR_IRQ_BBUS_START + 1) /* @Footnote 1 */
#define IRQ_BBUS_S1RXE		(NR_IRQ_BBUS_START + 2)
#define IRQ_BBUS_S1TXE		(NR_IRQ_BBUS_START + 3)
#define IRQ_BBUS_S2RXE		(NR_IRQ_BBUS_START + 4)
#define IRQ_BBUS_S2TXE		(NR_IRQ_BBUS_START + 5)
#define IRQ_BBUS_S3RXE		(NR_IRQ_BBUS_START + 6)
#define IRQ_BBUS_S3TXE		(NR_IRQ_BBUS_START + 7)
#define IRQ_BBUS_S4RXE		(NR_IRQ_BBUS_START + 8)
#define IRQ_BBUS_S4TXE		(NR_IRQ_BBUS_START + 9)
#define IRQ_BBUS_I2C		(NR_IRQ_BBUS_START + 10)
#define IRQ_BBUS_1284		(NR_IRQ_BBUS_START + 11)
#define IRQ_BBUS_UTIL		(NR_IRQ_BBUS_START + 12)
#define IRQ_BBUS_DMA1		(NR_IRQ_BBUS_START + 24)
#define IRQ_BBUS_DMA2		(NR_IRQ_BBUS_START + 25)

#define NR_IRQ_BBUS_DMA_START	(IRQ_BBUS_DMA2+1)
/* BBUS DMA Interrupts [1], chap 9 */

#define IRQ_BBUS_DMA_1		(NR_IRQ_BBUS_DMA_START)
#define IRQ_BBUS_DMA_2		(NR_IRQ_BBUS_DMA_START + 1)
#define IRQ_BBUS_DMA_3		(NR_IRQ_BBUS_DMA_START + 2)
#define IRQ_BBUS_DMA_4		(NR_IRQ_BBUS_DMA_START + 3)
#define IRQ_BBUS_DMA_5		(NR_IRQ_BBUS_DMA_START + 4)
#define IRQ_BBUS_DMA_6		(NR_IRQ_BBUS_DMA_START + 5)
#define IRQ_BBUS_DMA_7		(NR_IRQ_BBUS_DMA_START + 6)
#define IRQ_BBUS_DMA_8		(NR_IRQ_BBUS_DMA_START + 7)
#define IRQ_BBUS_DMA_9		(NR_IRQ_BBUS_DMA_START + 8)
#define IRQ_BBUS_DMA_10		(NR_IRQ_BBUS_DMA_START + 9)
#define IRQ_BBUS_DMA_11		(NR_IRQ_BBUS_DMA_START + 10)
#define IRQ_BBUS_DMA_12		(NR_IRQ_BBUS_DMA_START + 11)
#define IRQ_BBUS_DMA_13		(NR_IRQ_BBUS_DMA_START + 12)
#define IRQ_BBUS_DMA_14		(NR_IRQ_BBUS_DMA_START + 13)
#define IRQ_BBUS_DMA_15		(NR_IRQ_BBUS_DMA_START + 14)
#define IRQ_BBUS_DMA_16		(NR_IRQ_BBUS_DMA_START + 15)

#define NR_IRQ_USB_START	(IRQ_BBUS_DMA_16+1)
/* USB Interrupts [1], chap 17 */

#define IRQ_USB_DMA		(NR_IRQ_USB_START)
#define IRQ_USB_HOST		(NR_IRQ_USB_START + 1)
#define IRQ_USB_FIFO		(NR_IRQ_USB_START + 2)

#define NR_IRQ_EXTINT0_START	(IRQ_USB_FIFO + 1)

#define IRQ_IDE			(NR_IRQ_EXTINT0_START + 0)
#define IRQ_RTC			(NR_IRQ_EXTINT0_START + 1)

#define EXTINT0_IRQS		0x00000003

#define NR_IRQS			(IRQ_RTC + 1)

#endif /* __ASM_ARM_ARCH_IRQS_H */
